@@ -1,21 +1,11 @@
 // Copyright The OpenTelemetry Authors
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package metric
 
 import (
 	"context"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"strings"
@@ -23,6 +13,7 @@ import (
 	"testing"
 
 	"github.com/go-logr/logr"
+	"github.com/go-logr/logr/funcr"
 	"github.com/go-logr/logr/testr"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -168,8 +159,22 @@ func TestCallbackUnregisterConcurrency(t *testing.T) {
 
 // Instruments should produce correct ResourceMetrics.
 func TestMeterCreatesInstruments(t *testing.T) {
-	attrs := attribute.NewSet(attribute.String("name", "alice"))
-	opt := metric.WithAttributeSet(attrs)
+	// The synchronous measurement methods must ignore the context cancellation.
+	ctx, cancel := context.WithCancel(context.Background())
+	cancel()
+
+	alice := attribute.NewSet(
+		attribute.String("name", "Alice"),
+		attribute.Bool("admin", true),
+	)
+	optAlice := metric.WithAttributeSet(alice)
+
+	bob := attribute.NewSet(
+		attribute.String("name", "Bob"),
+		attribute.Bool("admin", false),
+	)
+	optBob := metric.WithAttributeSet(bob)
+
 	testCases := []struct {
 		name string
 		fn   func(*testing.T, metric.Meter)
@@ -178,11 +183,17 @@ func TestMeterCreatesInstruments(t *testing.T) {
 		{
 			name: "ObservableInt64Count",
 			fn: func(t *testing.T, m metric.Meter) {
-				cback := func(_ context.Context, o metric.Int64Observer) error {
-					o.Observe(4, opt)
-					return nil
-				}
-				ctr, err := m.Int64ObservableCounter("aint", metric.WithInt64Callback(cback))
+				ctr, err := m.Int64ObservableCounter(
+					"aint",
+					metric.WithInt64Callback(func(_ context.Context, o metric.Int64Observer) error {
+						o.Observe(4, optAlice)
+						return nil
+					}),
+					metric.WithInt64Callback(func(_ context.Context, o metric.Int64Observer) error {
+						o.Observe(5, optBob)
+						return nil
+					}),
+				)
 				assert.NoError(t, err)
 				_, err = m.RegisterCallback(func(_ context.Context, o metric.Observer) error {
 					o.ObserveInt64(ctr, 3)
@@ -196,7 +207,8 @@ func TestMeterCreatesInstruments(t *testing.T) {
 					Temporality: metricdata.CumulativeTemporality,
 					IsMonotonic: true,
 					DataPoints: []metricdata.DataPoint[int64]{
-						{Attributes: attrs, Value: 4},
+						{Attributes: alice, Value: 4},
+						{Attributes: bob, Value: 5},
 						{Value: 3},
 					},
 				},
@@ -205,11 +217,17 @@ func TestMeterCreatesInstruments(t *testing.T) {
 		{
 			name: "ObservableInt64UpDownCount",
 			fn: func(t *testing.T, m metric.Meter) {
-				cback := func(_ context.Context, o metric.Int64Observer) error {
-					o.Observe(4, opt)
-					return nil
-				}
-				ctr, err := m.Int64ObservableUpDownCounter("aint", metric.WithInt64Callback(cback))
+				ctr, err := m.Int64ObservableUpDownCounter(
+					"aint",
+					metric.WithInt64Callback(func(_ context.Context, o metric.Int64Observer) error {
+						o.Observe(4, optAlice)
+						return nil
+					}),
+					metric.WithInt64Callback(func(_ context.Context, o metric.Int64Observer) error {
+						o.Observe(5, optBob)
+						return nil
+					}),
+				)
 				assert.NoError(t, err)
 				_, err = m.RegisterCallback(func(_ context.Context, o metric.Observer) error {
 					o.ObserveInt64(ctr, 11)
@@ -223,7 +241,8 @@ func TestMeterCreatesInstruments(t *testing.T) {
 					Temporality: metricdata.CumulativeTemporality,
 					IsMonotonic: false,
 					DataPoints: []metricdata.DataPoint[int64]{
-						{Attributes: attrs, Value: 4},
+						{Attributes: alice, Value: 4},
+						{Attributes: bob, Value: 5},
 						{Value: 11},
 					},
 				},
@@ -232,11 +251,17 @@ func TestMeterCreatesInstruments(t *testing.T) {
 		{
 			name: "ObservableInt64Gauge",
 			fn: func(t *testing.T, m metric.Meter) {
-				cback := func(_ context.Context, o metric.Int64Observer) error {
-					o.Observe(4, opt)
-					return nil
-				}
-				gauge, err := m.Int64ObservableGauge("agauge", metric.WithInt64Callback(cback))
+				gauge, err := m.Int64ObservableGauge(
+					"agauge",
+					metric.WithInt64Callback(func(_ context.Context, o metric.Int64Observer) error {
+						o.Observe(4, optAlice)
+						return nil
+					}),
+					metric.WithInt64Callback(func(_ context.Context, o metric.Int64Observer) error {
+						o.Observe(5, optBob)
+						return nil
+					}),
+				)
 				assert.NoError(t, err)
 				_, err = m.RegisterCallback(func(_ context.Context, o metric.Observer) error {
 					o.ObserveInt64(gauge, 11)
@@ -248,7 +273,8 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				Name: "agauge",
 				Data: metricdata.Gauge[int64]{
 					DataPoints: []metricdata.DataPoint[int64]{
-						{Attributes: attrs, Value: 4},
+						{Attributes: alice, Value: 4},
+						{Attributes: bob, Value: 5},
 						{Value: 11},
 					},
 				},
@@ -257,11 +283,17 @@ func TestMeterCreatesInstruments(t *testing.T) {
 		{
 			name: "ObservableFloat64Count",
 			fn: func(t *testing.T, m metric.Meter) {
-				cback := func(_ context.Context, o metric.Float64Observer) error {
-					o.Observe(4, opt)
-					return nil
-				}
-				ctr, err := m.Float64ObservableCounter("afloat", metric.WithFloat64Callback(cback))
+				ctr, err := m.Float64ObservableCounter(
+					"afloat",
+					metric.WithFloat64Callback(func(_ context.Context, o metric.Float64Observer) error {
+						o.Observe(4, optAlice)
+						return nil
+					}),
+					metric.WithFloat64Callback(func(_ context.Context, o metric.Float64Observer) error {
+						o.Observe(5, optBob)
+						return nil
+					}),
+				)
 				assert.NoError(t, err)
 				_, err = m.RegisterCallback(func(_ context.Context, o metric.Observer) error {
 					o.ObserveFloat64(ctr, 3)
@@ -275,7 +307,8 @@ func TestMeterCreatesInstruments(t *testing.T) {
 					Temporality: metricdata.CumulativeTemporality,
 					IsMonotonic: true,
 					DataPoints: []metricdata.DataPoint[float64]{
-						{Attributes: attrs, Value: 4},
+						{Attributes: alice, Value: 4},
+						{Attributes: bob, Value: 5},
 						{Value: 3},
 					},
 				},
@@ -284,11 +317,17 @@ func TestMeterCreatesInstruments(t *testing.T) {
 		{
 			name: "ObservableFloat64UpDownCount",
 			fn: func(t *testing.T, m metric.Meter) {
-				cback := func(_ context.Context, o metric.Float64Observer) error {
-					o.Observe(4, opt)
-					return nil
-				}
-				ctr, err := m.Float64ObservableUpDownCounter("afloat", metric.WithFloat64Callback(cback))
+				ctr, err := m.Float64ObservableUpDownCounter(
+					"afloat",
+					metric.WithFloat64Callback(func(_ context.Context, o metric.Float64Observer) error {
+						o.Observe(4, optAlice)
+						return nil
+					}),
+					metric.WithFloat64Callback(func(_ context.Context, o metric.Float64Observer) error {
+						o.Observe(5, optBob)
+						return nil
+					}),
+				)
 				assert.NoError(t, err)
 				_, err = m.RegisterCallback(func(_ context.Context, o metric.Observer) error {
 					o.ObserveFloat64(ctr, 11)
@@ -302,7 +341,8 @@ func TestMeterCreatesInstruments(t *testing.T) {
 					Temporality: metricdata.CumulativeTemporality,
 					IsMonotonic: false,
 					DataPoints: []metricdata.DataPoint[float64]{
-						{Attributes: attrs, Value: 4},
+						{Attributes: alice, Value: 4},
+						{Attributes: bob, Value: 5},
 						{Value: 11},
 					},
 				},
@@ -311,11 +351,17 @@ func TestMeterCreatesInstruments(t *testing.T) {
 		{
 			name: "ObservableFloat64Gauge",
 			fn: func(t *testing.T, m metric.Meter) {
-				cback := func(_ context.Context, o metric.Float64Observer) error {
-					o.Observe(4, opt)
-					return nil
-				}
-				gauge, err := m.Float64ObservableGauge("agauge", metric.WithFloat64Callback(cback))
+				gauge, err := m.Float64ObservableGauge(
+					"agauge",
+					metric.WithFloat64Callback(func(_ context.Context, o metric.Float64Observer) error {
+						o.Observe(4, optAlice)
+						return nil
+					}),
+					metric.WithFloat64Callback(func(_ context.Context, o metric.Float64Observer) error {
+						o.Observe(5, optBob)
+						return nil
+					}),
+				)
 				assert.NoError(t, err)
 				_, err = m.RegisterCallback(func(_ context.Context, o metric.Observer) error {
 					o.ObserveFloat64(gauge, 11)
@@ -327,7 +373,8 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				Name: "agauge",
 				Data: metricdata.Gauge[float64]{
 					DataPoints: []metricdata.DataPoint[float64]{
-						{Attributes: attrs, Value: 4},
+						{Attributes: alice, Value: 4},
+						{Attributes: bob, Value: 5},
 						{Value: 11},
 					},
 				},
@@ -340,7 +387,7 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				ctr, err := m.Int64Counter("sint")
 				assert.NoError(t, err)
 
-				ctr.Add(context.Background(), 3)
+				ctr.Add(ctx, 3)
 			},
 			want: metricdata.Metrics{
 				Name: "sint",
@@ -359,7 +406,7 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				ctr, err := m.Int64UpDownCounter("sint")
 				assert.NoError(t, err)
 
-				ctr.Add(context.Background(), 11)
+				ctr.Add(ctx, 11)
 			},
 			want: metricdata.Metrics{
 				Name: "sint",
@@ -378,7 +425,7 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				gauge, err := m.Int64Histogram("histogram")
 				assert.NoError(t, err)
 
-				gauge.Record(context.Background(), 7)
+				gauge.Record(ctx, 7)
 			},
 			want: metricdata.Metrics{
 				Name: "histogram",
@@ -404,7 +451,7 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				ctr, err := m.Float64Counter("sfloat")
 				assert.NoError(t, err)
 
-				ctr.Add(context.Background(), 3)
+				ctr.Add(ctx, 3)
 			},
 			want: metricdata.Metrics{
 				Name: "sfloat",
@@ -423,7 +470,7 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				ctr, err := m.Float64UpDownCounter("sfloat")
 				assert.NoError(t, err)
 
-				ctr.Add(context.Background(), 11)
+				ctr.Add(ctx, 11)
 			},
 			want: metricdata.Metrics{
 				Name: "sfloat",
@@ -442,7 +489,7 @@ func TestMeterCreatesInstruments(t *testing.T) {
 				gauge, err := m.Float64Histogram("histogram")
 				assert.NoError(t, err)
 
-				gauge.Record(context.Background(), 7)
+				gauge.Record(ctx, 7)
 			},
 			want: metricdata.Metrics{
 				Name: "histogram",
@@ -759,7 +806,7 @@ func TestMeterCreatesInstrumentsValidations(t *testing.T) {
 		t.Run(tt.name, func(t *testing.T) {
 			m := NewMeterProvider().Meter("testInstruments")
 			err := tt.fn(t, m)
-			assert.Equal(t, err, tt.wantErr)
+			assert.Equal(t, tt.wantErr, err)
 		})
 	}
 }
@@ -994,7 +1041,7 @@ func TestGlobalInstRegisterCallback(t *testing.T) {
 	got := metricdata.ResourceMetrics{}
 	err = rdr.Collect(context.Background(), &got)
 	assert.NoError(t, err)
-	assert.Lenf(t, l.messages, 0, "Warnings and errors logged:\n%s", l)
+	assert.Emptyf(t, l.messages, "Warnings and errors logged:\n%s", l)
 	metricdatatest.AssertEqual(t, metricdata.ResourceMetrics{
 		ScopeMetrics: []metricdata.ScopeMetrics{
 			{
@@ -1124,13 +1171,13 @@ func TestUnregisterUnregisters(t *testing.T) {
 	int64Gauge, err := m.Int64ObservableGauge("int64.gauge")
 	require.NoError(t, err)
 
-	floag64Counter, err := m.Float64ObservableCounter("floag64.counter")
+	float64Counter, err := m.Float64ObservableCounter("float64.counter")
 	require.NoError(t, err)
 
-	floag64UpDownCounter, err := m.Float64ObservableUpDownCounter("floag64.up_down_counter")
+	float64UpDownCounter, err := m.Float64ObservableUpDownCounter("float64.up_down_counter")
 	require.NoError(t, err)
 
-	floag64Gauge, err := m.Float64ObservableGauge("floag64.gauge")
+	float64Gauge, err := m.Float64ObservableGauge("float64.gauge")
 	require.NoError(t, err)
 
 	var called bool
@@ -1142,9 +1189,9 @@ func TestUnregisterUnregisters(t *testing.T) {
 		int64Counter,
 		int64UpDownCounter,
 		int64Gauge,
-		floag64Counter,
-		floag64UpDownCounter,
-		floag64Gauge,
+		float64Counter,
+		float64UpDownCounter,
+		float64Gauge,
 	)
 	require.NoError(t, err)
 
@@ -1178,13 +1225,13 @@ func TestRegisterCallbackDropAggregations(t *testing.T) {
 	int64Gauge, err := m.Int64ObservableGauge("int64.gauge")
 	require.NoError(t, err)
 
-	floag64Counter, err := m.Float64ObservableCounter("floag64.counter")
+	float64Counter, err := m.Float64ObservableCounter("float64.counter")
 	require.NoError(t, err)
 
-	floag64UpDownCounter, err := m.Float64ObservableUpDownCounter("floag64.up_down_counter")
+	float64UpDownCounter, err := m.Float64ObservableUpDownCounter("float64.up_down_counter")
 	require.NoError(t, err)
 
-	floag64Gauge, err := m.Float64ObservableGauge("floag64.gauge")
+	float64Gauge, err := m.Float64ObservableGauge("float64.gauge")
 	require.NoError(t, err)
 
 	var called bool
@@ -1196,9 +1243,9 @@ func TestRegisterCallbackDropAggregations(t *testing.T) {
 		int64Counter,
 		int64UpDownCounter,
 		int64Gauge,
-		floag64Counter,
-		floag64UpDownCounter,
-		floag64Gauge,
+		float64Counter,
+		float64UpDownCounter,
+		float64Gauge,
 	)
 	require.NoError(t, err)
 
@@ -1207,7 +1254,7 @@ func TestRegisterCallbackDropAggregations(t *testing.T) {
 	require.NoError(t, err)
 
 	assert.False(t, called, "callback called for all drop instruments")
-	assert.Len(t, data.ScopeMetrics, 0, "metrics exported for drop instruments")
+	assert.Empty(t, data.ScopeMetrics, "metrics exported for drop instruments")
 }
 
 func TestAttributeFilter(t *testing.T) {
@@ -1579,7 +1626,12 @@ func TestObservableExample(t *testing.T) {
 		process1001 = attribute.NewSet(processID1001)
 	)
 
-	setup := func(t *testing.T, temp metricdata.Temporality) (map[attribute.Set]int64, func(*testing.T), *metricdata.ScopeMetrics, *int64, *int64, *int64) {
+	type observation struct {
+		attrs attribute.Set
+		value int64
+	}
+
+	setup := func(t *testing.T, temp metricdata.Temporality) (map[attribute.Distinct]observation, func(*testing.T), *metricdata.ScopeMetrics, *int64, *int64, *int64) {
 		t.Helper()
 
 		const (
@@ -1589,7 +1641,8 @@ func TestObservableExample(t *testing.T) {
 		)
 
 		selector := func(InstrumentKind) metricdata.Temporality { return temp }
-		reader := NewManualReader(WithTemporalitySelector(selector))
+		reader1 := NewManualReader(WithTemporalitySelector(selector))
+		reader2 := NewManualReader(WithTemporalitySelector(selector))
 
 		allowAll := attribute.NewDenyKeysFilter()
 		noFiltered := NewView(Instrument{Name: instName}, Stream{Name: instName, AttributeFilter: allowAll})
@@ -1597,14 +1650,14 @@ func TestObservableExample(t *testing.T) {
 		filter := attribute.NewDenyKeysFilter("tid")
 		filtered := NewView(Instrument{Name: instName}, Stream{Name: filteredStream, AttributeFilter: filter})
 
-		mp := NewMeterProvider(WithReader(reader), WithView(noFiltered, filtered))
+		mp := NewMeterProvider(WithReader(reader1), WithReader(reader2), WithView(noFiltered, filtered))
 		meter := mp.Meter(scopeName)
 
-		observations := make(map[attribute.Set]int64)
+		observations := make(map[attribute.Distinct]observation)
 		_, err := meter.Int64ObservableCounter(instName, metric.WithInt64Callback(
 			func(_ context.Context, o metric.Int64Observer) error {
-				for attrSet, val := range observations {
-					o.Observe(val, metric.WithAttributeSet(attrSet))
+				for _, val := range observations {
+					o.Observe(val.value, metric.WithAttributeSet(val.attrs))
 				}
 				return nil
 			},
@@ -1644,7 +1697,13 @@ func TestObservableExample(t *testing.T) {
 		collect := func(t *testing.T) {
 			t.Helper()
 			got := metricdata.ResourceMetrics{}
-			err := reader.Collect(context.Background(), &got)
+			err := reader1.Collect(context.Background(), &got)
+			require.NoError(t, err)
+			require.Len(t, got.ScopeMetrics, 1)
+			metricdatatest.AssertEqual(t, *want, got.ScopeMetrics[0], metricdatatest.IgnoreTimestamp())
+
+			got = metricdata.ResourceMetrics{}
+			err = reader2.Collect(context.Background(), &got)
 			require.NoError(t, err)
 			require.Len(t, got.ScopeMetrics, 1)
 			metricdatatest.AssertEqual(t, *want, got.ScopeMetrics[0], metricdatatest.IgnoreTimestamp())
@@ -1660,8 +1719,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T0, T1]:
 		//     pid = 1001, tid = 1, #PF = 50
 		//     pid = 1001, tid = 2, #PF = 30
-		observations[thread1] = 50
-		observations[thread2] = 30
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 50}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 30}
 
 		*wantFiltered = 80
 		*wantThread1 = 50
@@ -1672,8 +1731,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T1, T2]:
 		//     pid = 1001, tid = 1, #PF = 53
 		//     pid = 1001, tid = 2, #PF = 38
-		observations[thread1] = 53
-		observations[thread2] = 38
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 53}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 38}
 
 		*wantFiltered = 91
 		*wantThread1 = 53
@@ -1684,8 +1743,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T2, T3]
 		//     pid = 1001, tid = 1, #PF = 56
 		//     pid = 1001, tid = 2, #PF = 42
-		observations[thread1] = 56
-		observations[thread2] = 42
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 56}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 42}
 
 		*wantFiltered = 98
 		*wantThread1 = 56
@@ -1696,8 +1755,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T3, T4]:
 		//     pid = 1001, tid = 1, #PF = 60
 		//     pid = 1001, tid = 2, #PF = 47
-		observations[thread1] = 60
-		observations[thread2] = 47
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 60}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 47}
 
 		*wantFiltered = 107
 		*wantThread1 = 60
@@ -1709,9 +1768,9 @@ func TestObservableExample(t *testing.T) {
 		//     thread 1 died, thread 3 started
 		//     pid = 1001, tid = 2, #PF = 53
 		//     pid = 1001, tid = 3, #PF = 5
-		delete(observations, thread1)
-		observations[thread2] = 53
-		observations[thread3] = 5
+		delete(observations, thread1.Equivalent())
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 53}
+		observations[thread3.Equivalent()] = observation{attrs: thread3, value: 5}
 
 		*wantFiltered = 58
 		want.Metrics[1].Data = metricdata.Sum[int64]{
@@ -1734,8 +1793,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T0, T1]:
 		//     pid = 1001, tid = 1, #PF = 50
 		//     pid = 1001, tid = 2, #PF = 30
-		observations[thread1] = 50
-		observations[thread2] = 30
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 50}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 30}
 
 		*wantFiltered = 80
 		*wantThread1 = 50
@@ -1746,8 +1805,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T1, T2]:
 		//     pid = 1001, tid = 1, #PF = 53
 		//     pid = 1001, tid = 2, #PF = 38
-		observations[thread1] = 53
-		observations[thread2] = 38
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 53}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 38}
 
 		*wantFiltered = 11
 		*wantThread1 = 3
@@ -1758,8 +1817,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T2, T3]
 		//     pid = 1001, tid = 1, #PF = 56
 		//     pid = 1001, tid = 2, #PF = 42
-		observations[thread1] = 56
-		observations[thread2] = 42
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 56}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 42}
 
 		*wantFiltered = 7
 		*wantThread1 = 3
@@ -1770,8 +1829,8 @@ func TestObservableExample(t *testing.T) {
 		// During the time range (T3, T4]:
 		//     pid = 1001, tid = 1, #PF = 60
 		//     pid = 1001, tid = 2, #PF = 47
-		observations[thread1] = 60
-		observations[thread2] = 47
+		observations[thread1.Equivalent()] = observation{attrs: thread1, value: 60}
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 47}
 
 		*wantFiltered = 9
 		*wantThread1 = 4
@@ -1783,9 +1842,9 @@ func TestObservableExample(t *testing.T) {
 		//     thread 1 died, thread 3 started
 		//     pid = 1001, tid = 2, #PF = 53
 		//     pid = 1001, tid = 3, #PF = 5
-		delete(observations, thread1)
-		observations[thread2] = 53
-		observations[thread3] = 5
+		delete(observations, thread1.Equivalent())
+		observations[thread2.Equivalent()] = observation{attrs: thread2, value: 53}
+		observations[thread3.Equivalent()] = observation{attrs: thread3, value: 5}
 
 		*wantFiltered = -49
 		want.Metrics[1].Data = metricdata.Sum[int64]{
@@ -2053,3 +2112,353 @@ func TestHistogramBucketPrecedenceOrdering(t *testing.T) {
 		})
 	}
 }
+
+func TestObservableDropAggregation(t *testing.T) {
+	const (
+		intPrefix         = "observable.int64."
+		intCntName        = "observable.int64.counter"
+		intUDCntName      = "observable.int64.up.down.counter"
+		intGaugeName      = "observable.int64.gauge"
+		floatPrefix       = "observable.float64."
+		floatCntName      = "observable.float64.counter"
+		floatUDCntName    = "observable.float64.up.down.counter"
+		floatGaugeName    = "observable.float64.gauge"
+		unregPrefix       = "unregistered.observable."
+		unregIntCntName   = "unregistered.observable.int64.counter"
+		unregFloatCntName = "unregistered.observable.float64.counter"
+	)
+
+	type log struct {
+		name   string
+		number string
+	}
+
+	testcases := []struct {
+		name            string
+		views           []View
+		wantObservables []string
+		wantUnregLogs   []log
+	}{
+		{
+			name:  "default",
+			views: nil,
+			wantObservables: []string{
+				intCntName, intUDCntName, intGaugeName,
+				floatCntName, floatUDCntName, floatGaugeName,
+			},
+			wantUnregLogs: []log{
+				{
+					name:   unregIntCntName,
+					number: "int64",
+				},
+				{
+					name:   unregFloatCntName,
+					number: "float64",
+				},
+			},
+		},
+		{
+			name: "drop all metrics",
+			views: []View{
+				func(i Instrument) (Stream, bool) {
+					return Stream{Aggregation: AggregationDrop{}}, true
+				},
+			},
+			wantObservables: nil,
+			wantUnregLogs:   nil,
+		},
+		{
+			name: "drop float64 observable",
+			views: []View{
+				func(i Instrument) (Stream, bool) {
+					if strings.HasPrefix(i.Name, floatPrefix) {
+						return Stream{Aggregation: AggregationDrop{}}, true
+					}
+					return Stream{}, false
+				},
+			},
+			wantObservables: []string{
+				intCntName, intUDCntName, intGaugeName,
+			},
+			wantUnregLogs: []log{
+				{
+					name:   unregIntCntName,
+					number: "int64",
+				},
+				{
+					name:   unregFloatCntName,
+					number: "float64",
+				},
+			},
+		},
+		{
+			name: "drop int64 observable",
+			views: []View{
+				func(i Instrument) (Stream, bool) {
+					if strings.HasPrefix(i.Name, intPrefix) {
+						return Stream{Aggregation: AggregationDrop{}}, true
+					}
+					return Stream{}, false
+				},
+			},
+			wantObservables: []string{
+				floatCntName, floatUDCntName, floatGaugeName,
+			},
+			wantUnregLogs: []log{
+				{
+					name:   unregIntCntName,
+					number: "int64",
+				},
+				{
+					name:   unregFloatCntName,
+					number: "float64",
+				},
+			},
+		},
+		{
+			name: "drop unregistered observable",
+			views: []View{
+				func(i Instrument) (Stream, bool) {
+					if strings.HasPrefix(i.Name, unregPrefix) {
+						return Stream{Aggregation: AggregationDrop{}}, true
+					}
+					return Stream{}, false
+				},
+			},
+			wantObservables: []string{
+				intCntName, intUDCntName, intGaugeName,
+				floatCntName, floatUDCntName, floatGaugeName,
+			},
+			wantUnregLogs: nil,
+		},
+	}
+	for _, tt := range testcases {
+		t.Run(tt.name, func(t *testing.T) {
+			var unregLogs []log
+			otel.SetLogger(
+				funcr.NewJSON(
+					func(obj string) {
+						var entry map[string]interface{}
+						_ = json.Unmarshal([]byte(obj), &entry)
+
+						// All unregistered observables should log `errUnregObserver` error.
+						// A observable with drop aggregation is also unregistered,
+						// however this is expected and should not log an error.
+						assert.Equal(t, errUnregObserver.Error(), entry["error"])
+
+						unregLogs = append(unregLogs, log{
+							name:   fmt.Sprintf("%v", entry["name"]),
+							number: fmt.Sprintf("%v", entry["number"]),
+						})
+					},
+					funcr.Options{Verbosity: 0},
+				),
+			)
+			defer otel.SetLogger(logr.Discard())
+
+			reader := NewManualReader()
+			meter := NewMeterProvider(WithView(tt.views...), WithReader(reader)).Meter("TestObservableDropAggregation")
+
+			intCnt, err := meter.Int64ObservableCounter(intCntName)
+			require.NoError(t, err)
+			intUDCnt, err := meter.Int64ObservableUpDownCounter(intUDCntName)
+			require.NoError(t, err)
+			intGaugeCnt, err := meter.Int64ObservableGauge(intGaugeName)
+			require.NoError(t, err)
+
+			floatCnt, err := meter.Float64ObservableCounter(floatCntName)
+			require.NoError(t, err)
+			floatUDCnt, err := meter.Float64ObservableUpDownCounter(floatUDCntName)
+			require.NoError(t, err)
+			floatGaugeCnt, err := meter.Float64ObservableGauge(floatGaugeName)
+			require.NoError(t, err)
+
+			unregIntCnt, err := meter.Int64ObservableCounter(unregIntCntName)
+			require.NoError(t, err)
+			unregFloatCnt, err := meter.Float64ObservableCounter(unregFloatCntName)
+			require.NoError(t, err)
+
+			_, err = meter.RegisterCallback(
+				func(ctx context.Context, obs metric.Observer) error {
+					obs.ObserveInt64(intCnt, 1)
+					obs.ObserveInt64(intUDCnt, 1)
+					obs.ObserveInt64(intGaugeCnt, 1)
+					obs.ObserveFloat64(floatCnt, 1)
+					obs.ObserveFloat64(floatUDCnt, 1)
+					obs.ObserveFloat64(floatGaugeCnt, 1)
+					// We deliberately call observe to unregistered observables
+					obs.ObserveInt64(unregIntCnt, 1)
+					obs.ObserveFloat64(unregFloatCnt, 1)
+
+					return nil
+				},
+				intCnt, intUDCnt, intGaugeCnt,
+				floatCnt, floatUDCnt, floatGaugeCnt,
+				// We deliberately do not register `unregIntCnt` and `unregFloatCnt`
+				// to test that `errUnregObserver` is logged when observed by callback.
+			)
+			require.NoError(t, err)
+
+			var rm metricdata.ResourceMetrics
+			err = reader.Collect(context.Background(), &rm)
+			require.NoError(t, err)
+
+			if len(tt.wantObservables) == 0 {
+				require.Empty(t, rm.ScopeMetrics)
+				return
+			}
+
+			require.Len(t, rm.ScopeMetrics, 1)
+			require.Len(t, rm.ScopeMetrics[0].Metrics, len(tt.wantObservables))
+
+			for i, m := range rm.ScopeMetrics[0].Metrics {
+				assert.Equal(t, tt.wantObservables[i], m.Name)
+			}
+			assert.Equal(t, tt.wantUnregLogs, unregLogs)
+		})
+	}
+}
+
+func TestDuplicateInstrumentCreation(t *testing.T) {
+	for _, tt := range []struct {
+		desc             string
+		createInstrument func(metric.Meter) error
+	}{
+		{
+			desc: "Int64ObservableCounter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Int64ObservableCounter("observable.int64.counter")
+				return err
+			},
+		},
+		{
+			desc: "Int64ObservableUpDownCounter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Int64ObservableUpDownCounter("observable.int64.up.down.counter")
+				return err
+			},
+		},
+		{
+			desc: "Int64ObservableGauge",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Int64ObservableGauge("observable.int64.gauge")
+				return err
+			},
+		},
+		{
+			desc: "Float64ObservableCounter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Float64ObservableCounter("observable.float64.counter")
+				return err
+			},
+		},
+		{
+			desc: "Float64ObservableUpDownCounter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Float64ObservableUpDownCounter("observable.float64.up.down.counter")
+				return err
+			},
+		},
+		{
+			desc: "Float64ObservableGauge",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Float64ObservableGauge("observable.float64.gauge")
+				return err
+			},
+		},
+		{
+			desc: "Int64Counter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Int64Counter("sync.int64.counter")
+				return err
+			},
+		},
+		{
+			desc: "Int64UpDownCounter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Int64UpDownCounter("sync.int64.up.down.counter")
+				return err
+			},
+		},
+		{
+			desc: "Int64Histogram",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Int64Histogram("sync.int64.histogram")
+				return err
+			},
+		},
+		{
+			desc: "Float64Counter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Float64Counter("sync.float64.counter")
+				return err
+			},
+		},
+		{
+			desc: "Float64UpDownCounter",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Float64UpDownCounter("sync.float64.up.down.counter")
+				return err
+			},
+		},
+		{
+			desc: "Float64Histogram",
+			createInstrument: func(meter metric.Meter) error {
+				_, err := meter.Float64Histogram("sync.float64.histogram")
+				return err
+			},
+		},
+	} {
+		t.Run(tt.desc, func(t *testing.T) {
+			reader := NewManualReader()
+			defer func() {
+				require.NoError(t, reader.Shutdown(context.Background()))
+			}()
+
+			m := NewMeterProvider(WithReader(reader)).Meter("TestDuplicateInstrumentCreation")
+			for i := 0; i < 3; i++ {
+				require.NoError(t, tt.createInstrument(m))
+			}
+			internalMeter, ok := m.(*meter)
+			require.True(t, ok)
+			// check that multiple calls to create the same instrument only create 1 instrument
+			numInstruments := len(internalMeter.int64Insts.data) + len(internalMeter.float64Insts.data) + len(internalMeter.int64ObservableInsts.data) + len(internalMeter.float64ObservableInsts.data)
+			require.Equal(t, 1, numInstruments)
+		})
+	}
+}
+
+func TestMeterProviderDelegation(t *testing.T) {
+	meter := otel.Meter("go.opentelemetry.io/otel/metric/internal/global/meter_test")
+	otel.SetErrorHandler(otel.ErrorHandlerFunc(func(err error) { require.NoError(t, err) }))
+	for i := 0; i < 5; i++ {
+		int64Counter, err := meter.Int64ObservableCounter("observable.int64.counter")
+		require.NoError(t, err)
+		int64UpDownCounter, err := meter.Int64ObservableUpDownCounter("observable.int64.up.down.counter")
+		require.NoError(t, err)
+		int64Gauge, err := meter.Int64ObservableGauge("observable.int64.gauge")
+		require.NoError(t, err)
+		floatCounter, err := meter.Float64ObservableCounter("observable.float.counter")
+		require.NoError(t, err)
+		floatUpDownCounter, err := meter.Float64ObservableUpDownCounter("observable.float.up.down.counter")
+		require.NoError(t, err)
+		floatGauge, err := meter.Float64ObservableGauge("observable.float.gauge")
+		require.NoError(t, err)
+		_, err = meter.RegisterCallback(func(ctx context.Context, o metric.Observer) error {
+			o.ObserveInt64(int64Counter, int64(10))
+			o.ObserveInt64(int64UpDownCounter, int64(10))
+			o.ObserveInt64(int64Gauge, int64(10))
+
+			o.ObserveFloat64(floatCounter, float64(10))
+			o.ObserveFloat64(floatUpDownCounter, float64(10))
+			o.ObserveFloat64(floatGauge, float64(10))
+			return nil
+		}, int64Counter, int64UpDownCounter, int64Gauge, floatCounter, floatUpDownCounter, floatGauge)
+		require.NoError(t, err)
+	}
+	provider := NewMeterProvider()
+
+	assert.NotPanics(t, func() {
+		otel.SetMeterProvider(provider)
+	})
+}
