@@ -1,16 +1,5 @@
 // Copyright The OpenTelemetry Authors
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package trace_test
 
@@ -120,7 +109,7 @@ func TestSimpleSpanProcessorShutdown(t *testing.T) {
 	}
 }
 
-func TestSimpleSpanProcessorShutdownOnEndConcurrency(t *testing.T) {
+func TestSimpleSpanProcessorShutdownOnEndConcurrentSafe(t *testing.T) {
 	exporter := &testExporter{}
 	ssp := sdktrace.NewSimpleSpanProcessor(exporter)
 	tp := basicTracerProvider(t)
@@ -153,7 +142,7 @@ func TestSimpleSpanProcessorShutdownOnEndConcurrency(t *testing.T) {
 	<-done
 }
 
-func TestSimpleSpanProcessorShutdownOnEndRace(t *testing.T) {
+func TestSimpleSpanProcessorShutdownOnEndConcurrentSafe2(t *testing.T) {
 	exporter := &testExporter{}
 	ssp := sdktrace.NewSimpleSpanProcessor(exporter)
 	tp := basicTracerProvider(t)
